#!/bin/bash

# Constructs sinograms from a list of integrated diffraction patterns
# (diffractograms).

NR_JOBS=2

PEAK_FILE=$1 #'peak_730.dat'
DO_FLIP=true

[ -d sinograms ] || mkdir sinograms

trap control_c SIGINT

control_c()
# run if user hits control-c
{
  echo -en "\n*** Ouch! Exiting ***\n"
  exit $?
}

count=0
for diffractogram in "$@"; do 
  echo $diffractogram
  basename=$(basename $diffractogram .h5)
  name=SINOGRAM-$(echo $basename | sed s/DIFFTOMO-//)
  args="-o sinograms/$name $diffractogram.h5"
  if [ $DO_FLIP ]; then
      args="--flip $args"
  fi
  echo "Assembling $basename using $PEAK_FILE"
  xrdtoolkit-assemble --peak-file $PEAK_FILE $args &
  let count+=1
  [[ $((count%NR_JOBS)) -eq 0 ]] && wait # Limit to NR_JOBS concurrent jobs
done

wait
