#!/bin/env python

import h5py

from scipy import ndimage
import numpy as np

import xrdtoolkit
from xrdtoolkit import tomo, image, files

sinoFile = files.ImageFile('sinogram.h5')
f = h5py.File('sinogram.h5')
g = f['/xraylib/sinogram']
ref = f['/xraylib/sinogram_corrected']
rec = f['/xraylib/reconstructed']

keys = g.keys()
#keys_selection = [ keys[2],keys[3],keys[5],keys[8] ]
sinograms = dict({  i:j.value.squeeze() for i,j in g.items() })

sinograms_deint = dict({ o:tomo.sino_center(tomo.sino_deinterlace(j.value.squeeze())) for o,j in g.items() })
sinograms_sobel = dict({ o:ndimage.sobel(j,axis=0) for o,j in sinograms_deint.items() })
sinograms_corrected = dict({ key:image.value for key,image in ref.items() })

reconstructions = dict({ o:j.value.squeeze() for o,j in rec.items() })


offset_high = {}
offset_low = {}
mask_low = {}
mask_high = {}
mask = {}
median = {}
ret = {}

for key,img in sinograms_deint.items():
    a =  tomo.sino_remove_bragg_spots(img)
    offset_high.update({ key:a[1]})
    offset_low.update({ key:a[2]})
    mask_low.update({ key:a[3]})
    mask_high.update({ key:a[4]})
    mask.update({ key:a[3]+a[4]})
    median.update({ key:a[5]})
    ret.update({ key:a[6]})

for i in g.keys():
    sinoFile.saveImage( (~mask[i]) * sinograms_deint[i], '/xraylib/masked/%s' % (key,) )
    sinoFile.saveImage(mask[i], '/xraylib/mask/%s' % (key,) )

glass_img = reconstructions[u'gaussian (254.88,81.72)']
res = image.get_disc_parameters(glass_img)

# 'area': [269, 1817],
# 'diameter': [18.506794361362498, 48.098609676203644],


sinoFile.saveImage(res['labels'], '/xraylib/labeled_glass_phase')
