#!/bin/bash

NR_JOBS=1
PONI_FILE='180mm.poni'

[ -d diffractograms ] || mkdir diffractograms


trap control_c SIGINT

control_c()
# run if user hits control-c
{
  echo -en "\n*** Ouch! Exiting ***\n"
  exit $?
}

# Start parallell averaging of dark current
./average_dark.sh $@ &

for dir in "$@"; do
  name=$(basename $dir)
  dark=darkcurrent/DARK_$name.h5

  while [ ! -f $dark ]; do
      # Wait for dark current to be averaged
      echo $name' waiting for '$dark
      sleep 10
  done

  diffractogram=diffractograms/DIFFTOMO-$name.h5
  if [ ! -f $diffractogram ]; then
      echo "$name -- Integrating diffraction patterns"
      xrdtoolkit-integrate -p $PONI_FILE  --data-path $dir/$name --dark darkcurrent/DARK_$name.h5 -o $diffractogram --timings
  fi
done

