#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mex.h"

void mexFunction(int nl, mxArray *pl[], int nr, const mxArray *pr[]) {
  int      M, N, doReal;
  double  *k, *y, *c, *r, *F, dt, w, m0, m1, m2, n0, n1, n2, n3,
           rk, RK, *F0, *F1, *F2, *F3, cRK, crk, sRK, srk, spi;
  register int i, j;

  if (nr != 5) mexErrMsgTxt("Wrong number of input parameters");
  k = mxGetPr(pr[4]); doReal = k[0];
  k = mxGetPr(pr[0]); M = mxGetM(pr[0]);
  y = mxGetPr(pr[1]); if (M != mxGetM(pr[1])) mexErrMsgTxt("Doesn't match");
  c = mxGetPr(pr[2]); if (--M != mxGetM(pr[2])) mexErrMsgTxt("Doesn't match");
  if (mxGetN(pr[2]) != 3) mexErrMsgTxt("Wrong size of c");
  r = mxGetPr(pr[3]); N = mxGetN(pr[3]); spi = 1.0/sqrt(4*asin(1.0));
  pl[0] = mxCreateDoubleMatrix(1,N,mxREAL); F = mxGetPr(pl[0]);
  F0 = (double *) mxCalloc(N,sizeof(double));
  F1 = (double *) mxCalloc(N,sizeof(double));
  F2 = (double *) mxCalloc(N,sizeof(double));
  F3 = (double *) mxCalloc(N,sizeof(double));

  if (doReal) for (i = 0; i < M; i++) {
    dt = k[i+1] - k[i]; n0  = y[i]; n1 = c[i]; n2 = 2*c[i+M]; n3 = 6*c[i+2*M];
    m0  = n0 + dt*(n1 + dt*(c[i+M] + dt*c[i+2*M]));
    m1  = n1 + dt*(n2 + 3*dt*c[i+2*M]); m2 = n2 + dt*n3;
    for (j = 0; j < N; j++) {
      RK = r[j]*k[i+1]; rk = r[j]*k[i];
      sRK = sin(RK); cRK = cos(RK); srk = sin(rk); crk = cos(rk);
      F0[j] += m0*sRK - n0*srk; F1[j] += m1*cRK - n1*crk;
      F2[j] -= m2*sRK - n2*srk; F3[j] -= n3*(cRK - crk);
    }
  } else for (i = 0; i < M; i++) {
    dt = k[i+1] - k[i]; n0  = y[i]; n1 = c[i]; n2 = 2*c[i+M]; n3 = 6*c[i+2*M];
    m0  = n0 + dt*(n1 + dt*(c[i+M] + dt*c[i+2*M]));
    m1  = n1 + dt*(n2 + 3*dt*c[i+2*M]); m2 = n2 + dt*n3;
    for (j = 0; j < N; j++) {
      RK = r[j]*k[i+1]; rk = r[j]*k[i];
      sRK = sin(RK); cRK = cos(RK); srk = sin(rk); crk = cos(rk);
      F0[j] -= m0*cRK - n0*crk; F1[j] += m1*sRK - n1*srk;
      F2[j] += m2*cRK - n2*crk; F3[j] -= n3*(sRK - srk);
    }
  }
  for (j = 0; j < N; j++) if (r[j] != 0) {
    w = 1/r[j]; F[j] = (F0[j] + w*(F1[j] + w*(F2[j] + w*F3[j])))*w*spi;
  } else {
    if (doReal) {
      for (i = 0; i < M; i++) {
        dt = k[i+1] - k[i];
        F[j] = F[j] + dt*(y[i] + dt*(c[i]/2 + dt*(c[i+M]/3 + dt*c[i+2*M]/4)));
      }
      F[j] *= spi;
    } else F[j] = 0;
  }
  mxFree(F0); mxFree(F1); mxFree(F2); mxFree(F3);
}
