function Sc = inels(q,s);
% function S = inels(4 pi sin(Theta)/lambda, compound)
%   Calculates the inelastic scattering factor for given compound (see mpr).

global XrayTable
if nargin < 2, error('  Not enough input parameters'); end;
if length(XrayTable) == 0, load xraytable; end;

Sc = zeros(size(q)); spl = q(:)/(4*pi);
si = find(spl <= 1); bi = find(1 < spl);
f = zeros(size(q));
[Z,w] = strtoz(s); N = length(Z); St = 0;
for i = 1:N,
  tf = XrayTable(Z(i)).factor;
  if ~isempty(si), f(si) = interp1(tf.s,tf.S,spl(si),'cubic'); end;
  if ~isempty(bi),
    f(bi) = interp1(log(tf.s(2:end)),tf.S(2:end),log(spl(bi)),'cubic');
  end;
  St = St+w(i)*f;
end;
Sc(:) = St;
