function [S,G,r] = pdf(q,y,flt,cmp,Eo,Pl,r,bckg);
% function [S,G,r] = pdf(q, y, filter, compound, energy, polarization,
%                       [r, backgound]);
%    The filter can be any funcfion f(q) or a scalar. In the case of scalar
%    f(q) = exp(-(filter*q)^2/2) is used.
%    The default value of radial vector 'r' (no r is given or r = [])
%    is 0.1:0.1:20. The background is a matrix of vector of the size
%    length(q)xN, where N is the number of background components. If no
%    background is given the default background is 1/cos(tth)^2 (that
%    corresponds to a constant when flat area detector is used).

pow = 1;
if nargin < 6, r = (0.1:0.1:20)'; end;
if lengrh(r) < 1, r = (0.1:0.1:20)'; end;
if length(flt) == 1, flt = exp(-(q*flt).^2/2); end;

sam.compound = cmp;
sam.chi = 0; % deg
sam.thickness = 0.03; % cm
sam.density = 2.7 % g/cm^3


tth = 2*asin(q*constants('hc')/(4*pi*Eo)); tthd = deg(tth); cs = cos(tth);
if nargin < 7, bckg = 1./cs.^2; end;
f = formf(q,cmp); fe = forme(q,cmp); S = inels(q,cmp);
[K,Ec] = kn(Eo,tthd,Pl);
T = teff(sam,Eo,tthd).*thomson(tthd,Pl); K = teff(sam,Eo,tthd,Ec).*K;

A = [T.*f.^2+K.*S bckg]; s = q.^pow./f.^2;
As = A.*s(:,ones(1,size(A,2))); [u,s,v] = svd(As'*A,0);
s = diag(1./diag(s)); db = (v*s*u'); b = db*As'*y;
subplot(311); plot(q,y,q,A*b,q,A(:,2:end)*b(2:end)); grid;
xlabel('{\it q}  (1/A)'); ylabel('{\it I}  (au)');
y = (y-A(:,2:end)*b(2:end)-K.*S*b(1))/b(1)./T; S = (y-f.^2).*q./fe.^2;
subplot(312); plot(q,S,'.-',q,flt.*S,'.-'); grid;
xlabel('{\it q}  (1/A)'); ylabel('{\it S(q)}-1');
G = 2*sqrt(2/pi)*csft(q,S.*flt,r)';
subplot(313); plot(r,G,'.-'); grid; top;
xlabel('{\it r}  (A)'); ylabel('\it G(r)');


%%%%%%%%%%%%%%%

function T = teff(b,Eo,tth,E);
% function T = teff(sample object, Eo, TwoTheta, E);

x = b.chi; Thick = b.thickness; d = b.density;
cmp = b.compound; ian = x; oan = tth - x;
cin = cos(rad(ian)); cout = -cos(rad(oan));
cout = cout.*(1 - 2*(cin < 0)); cin = abs(cin);
mo = mpr(Eo,cmp)*d; if nargin < 4, m = mo; else, m = mpr(E,cmp)*d; end;
r = mo./cin + m./cout; warning off MATLAB:divideByZero
T = (1-exp(-r*Thick))./(r.*cin); warning on MATLAB:divideByZero
a = exp(m.*Thick./cout).*ones(size(r));
i = find(cout.*ones(size(T)) < 0); T(i) = T(i).*a(i);
a = a.*Thick./cin; i = find(abs(r) < 1e-6); T(i) = a(i);
